from enum import Enum
from typing import Optional, Callable, Any


class NetworkProtocol(Enum):
    HTTP = 'http'
    GRPC = 'grpc'
    GRPC_WEBSOCKET = 'grpc_websocket'


class NetworkEndpoint(Enum):
    LOG_EVENT = 'log_event'
    DOWNLOAD_CONFIG_SPECS = 'download_config_specs'
    GET_ID_LISTS = 'get_id_lists'
    ALL = 'all'


class IStreamingListeners:
    def __init__(self, on_update: Callable, on_error: Callable):
        self.on_update = on_update
        self.on_error = on_error



class IStatsigNetworkWorker:
    @property
    def type(self) -> NetworkProtocol:
        return NetworkProtocol.HTTP

    def is_pull_worker(self) -> bool:
        return True

    def get_dcs(self, on_complete: Callable, since_time: int = 0,
                log_on_exception: Optional[bool] = False, timeout: Optional[int] = None):
        pass

    def get_id_lists(self, on_complete: Callable, log_on_exception: Optional[bool] = False,
                     timeout: Optional[int] = None):
        pass

    def get_id_list(self, on_complete: Any, url, headers, log_on_exception=False):
        pass

    def log_events(self, payload, headers=None, log_on_exception=False, retry=0):
        pass

    def shutdown(self) -> None:
        pass


class IStatsigWebhookWorker:
    def start_listen_for_config_spec(self, listeners: IStreamingListeners) -> None:
        pass

    def start_listen_for_id_list(self, listeners: IStreamingListeners) -> None:
        pass
